#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division
from rayon import data, events
from netsa.data import nice, format
import time

def get_chart(tools, real_data, metadata):
    chart = tools.new_chart("square")
    
    # Scales
    xscale = tools.new_scale("catrange")
    yscale = tools.new_scale("linear")

    # Plot
    bar_plot = tools.new_plot("vbar")
    bar_plot.set_scales(x=xscale, y=yscale, bar_width=1,
                        bar_color=(0, 0, 255, 128),
                        border_width=0, border_color=(0, 0, 255, 128)) 
    bar_plot.set_data(x=real_data.column('date'),
                      y=real_data.column(metadata['show'][0]))
    bar_plot.set_metadata(unit_name=metadata['show'][0])

    chart.add_plot(bar_plot, "bars")

    # Borders
    x_lo, x_hi = (real_data.column('date').min(),
              real_data.column('date').max())
    x_num_ticks = min(len(real_data.column('date')), 12)
    xtick_tuples = ((x, time.strftime("%y-%m-%d:%H", x.timetuple()))
                    for x in nice.nice_time_ticks(x_lo, x_hi, x_num_ticks)[2])
    xborder = tools.new_border(
        'htick', scale=xscale, tick_tuples=xtick_tuples,
        tickmarker_args=dict(angle=45, halign='right', valign='center',),
                             #marker_type='none'),
                             tpad=.05, line_type=None)
    def formatfn(x):
        if metadata['show'][0] == 'bytes':
            units = "B"
        else:
            units = ""
        return format.num_prefix(x, units=units)
    # Associate the formatter function in the metadata so the callback
    # can use it.
    bar_plot.set_metadata(formatfn=formatfn)
    
    yborder = tools.new_border('vnice', scale=yscale, tick_formatter=formatfn,
                               rpad=.05, line_type=None,
                               tickmarker_args=dict(marker_type='none'))


    chart.add_bottom_border(xborder, height=.1)
    chart.add_left_border(yborder, width=.1)
        
    chart.add_padding("all", .05)

    # Background color and gridlines
    chart.set_chart_background("#fff")
    chart.set_plot_background("#ddd")

    # Use the outside ticks, minus the first and last
    y_grid = tools.new_gridlines('horizontal')
    grid_ticks = tuple(yborder.iter_tick_positions(0))[1:-1]
    y_grid.set_data(pos=data.Column(grid_ticks))
    y_grid.set_scales(pos=bar_plot.get_scale('y'),
                      line_color="white", line_style="solid",
                      line_width=2)
    chart.add_rear_gridlines(y_grid)

    # Title
    if metadata.has_key('title'):
        title = tools.new_border('hlabel', label=metadata['title'],
                                 font_size='large')
        chart.add_top_title(title, .1)

    return chart


class MyBarListener(events.EventListener):
    def __init__(self, component, viewpage):
        events.EventListener.__init__(self, component)
        self.viewpage = viewpage
    def on_tooltip_(self, target, evt):
        bars = target.bars_at(evt.x(), evt.y())
        if len(bars) > 0:
            bar = bars[0]
            formatfn = bar.extra_data['formatfn']
            evt.set_tooltip(formatfn(bar.ydata), 1)
    def on_mouse_left_dclick_(self, target, evt):
        bars = target.bars_at(evt.x(), evt.y())
        if len(bars) > 0 and self.viewpage is not None:
            my_plot = target
            bar = bars[0]
            unit_name = my_plot.get_metadata()['unit_name']
            # Example input to RefinementCallback:
            # {'stime': <datetime.datetime object>}
            self.viewpage.RefinementCallback(({'stime': bar.xdata},
                                              {unit_name: bar.ydata}))

            

def set_events(chart, event_source):
    viewpage = event_source.viewpage
    adapter = chart.get_event_source(event_source, activate=False)
    component = chart.get_plot('bars')
    listener = MyBarListener(component, viewpage)
    listener.register_with(adapter)




__all__ = ["get_data", "set_events"]
