#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
"""
Font support for unix and Mac, using fontconfig and freetype.

(The following is quoted from
http://fontconfig.org/fontconfig-user.html for reference. Note,
however, that you can use actual font names as font families. --pg)

Font Names

Fontconfig provides a textual representation for patterns that the
library can both accept and generate. The representation is in three
parts, first a list of family names, second a list of point sizes and
finally a list of additional properties:

	<families>-<point sizes>:<name1>=<values1>:<name2>=<values2>...
    

Values in a list are separated with commas. The name needn't include
either families or point sizes; they can be elided. In addition, there
are symbolic constants that simultaneously indicate both a name and a
value. Here are some examples:

  Name                            Meaning
  ----------------------------------------------------------
  Times-12                        12 point Times Roman
  Times-12:bold                   12 point Times Bold
  Courier:italic                  Courier Italic in the default size
  Monospace:matrix=1 .1 0 1       The users preferred monospace font
                                  with artificial obliquing
    

The '\', '-', ':' and ',' characters in family names must be preceeded
by a '\' character to avoid having them misinterpreted. Similarly,
values containing '\', '=', '_', ':' and ',' must also have them
preceeded by a '\' character. The '\' characters are stripped out of
the family name and values as the font name is read.
"""
import _fonts


font_families = {
    "sans_serif" : ["sans-serif"],
    "serif"      : ["serif"],
}
#font_families['default'] = font_families['sans_serif'] + font_families['serif']
font_families['default'] = font_families['serif'] + font_families['serif']

# Font weights, conveniently, include "normal", "light", and "bold",
# so no remapping is needed.

font_styles = {
    "normal" : "roman",
    "slant"  : "oblique",
    "italic" : "italic"
}


def switch_font(cairo_ctx, family, style, weight, size):
    # TODO: Only supporting fontconfig for now. At some point I will
    # implement a manual search-through-a-bunch-of-directories
    # solution, if it's warranted.

    # Use the family verbatim unless it's a reference to a predefined
    # family. Then use the predefined family.
    if family in font_families.keys():
        family = ",".join(font_families[family])

    fontconfig_name = "%s:size=%s:weight=%s:slant=%s" % (
        family, str(size), weight, font_styles.get(style, "roman"))

    _fonts.set_font_from_name(cairo_ctx, fontconfig_name)
        


__all__ = ["switch_font"]
