#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division
import math
import sys

import wx

import threading
import events
import backend

# Every backend has a name
name = "wx"

# The WX backend is an interactive backend -- you can register events
# and stuff.
interactive = True

# WX won't output to a file
output_to_file = False

# WX will output to memory
output_to_memory = True

#
# Fonts
#
font_families = {
    "default"    : wx.FONTFAMILY_DEFAULT,
    "serif"      : wx.FONTFAMILY_ROMAN,
    "sans-serif" : wx.FONTFAMILY_SWISS,
    "cursive"    : wx.FONTFAMILY_SWISS,
    "fantasy"    : wx.FONTFAMILY_SWISS,
    "monospace"  : wx.FONTFAMILY_SWISS,
}

font_styles = {
    "normal" : wx.FONTSTYLE_NORMAL,
    "oblique"  : wx.FONTSTYLE_SLANT,
    "italic" : wx.FONTSTYLE_ITALIC
}

#font_weights = {
#    "normal" : wx.FONTWEIGHT_NORMAL,
#    "light"  : wx.FONTWEIGHT_LIGHT,
#    "bold"   : wx.FONTWEIGHT_BOLD
#}

font_weights = {
    "normal" : wx.FONTWEIGHT_NORMAL,
    "bold"   : wx.FONTWEIGHT_BOLD,
    100      : wx.FONTWEIGHT_LIGHT,
    200      : wx.FONTWEIGHT_LIGHT,
    300      : wx.FONTWEIGHT_LIGHT,
    400      : wx.FONTWEIGHT_NORMAL,
    500      : wx.FONTWEIGHT_NORMAL,
    600      : wx.FONTWEIGHT_NORMAL,
    700      : wx.FONTWEIGHT_BOLD,
    800      : wx.FONTWEIGHT_BOLD,
    900      : wx.FONTWEIGHT_BOLD,
    "100"    : wx.FONTWEIGHT_LIGHT,
    "200"    : wx.FONTWEIGHT_LIGHT,
    "300"    : wx.FONTWEIGHT_LIGHT,
    "400"    : wx.FONTWEIGHT_NORMAL,
    "500"    : wx.FONTWEIGHT_NORMAL,
    "600"    : wx.FONTWEIGHT_NORMAL,
    "700"    : wx.FONTWEIGHT_BOLD,
    "800"    : wx.FONTWEIGHT_BOLD,
    "900"    : wx.FONTWEIGHT_BOLD
}

    
#
# Events
#
class WxRayonEvent(events.RayonEvent):
    """Proxy for wx event objects."""
    def __init__(self, wx_evt, x=None, y=None, x_offset=0, y_offset=0):
        self.wx_evt = wx_evt
        self.x_pos = x
        self.y_pos = y
        self.x_off = x_offset
        self.y_off = y_offset
        self.tooltip_msg = None
    def is_mouse_right_button_pressed(self):
        return self.wx_evt.RightIsDown()
    def is_mouse_left_button_pressed(self):
        return self.wx_evt.LeftIsDown()
    def is_mouse_moving(self):
        return self.wx_evt.Moving()
    def is_dragging(self):
        return self.wx_evt.Dragging()
    def set_tooltip(self, txt, weight=1):
        self.tooltip_msg = (txt, weight)
    def get_tooltip(self):
        if self.tooltip_msg is None:
            return (None, None)
        else:
            return self.tooltip_msg
    def canvas_x(self):
        """Return the X value relative to the left edge of the canvas."""
        if self.x_pos is None:
            return self.wx_evt.GetX() 
        else:
            return self.x_pos 
    def canvas_y(self):
        """Return the X value relative to the top of the canvas."""
        if self.y_pos is None:
            return self.wx_evt.GetY()
        else:
            return self.y_pos
    def x(self):
        """Return the X value relative to the left edge of the
        responsible component's bounding box."""
        return self.canvas_x() - self.x_off
    def y(self):
        """Return the Y value relative to the top of the responsible
        component's bounding box."""
        return self.canvas_y() - self.y_off


class Popper(object):
    """Handles tooltips. It's a Popper and not a Tooltip or some such
    because it doesn't use native tooltips, and because it's got
    different semantics. (E.g., it doesn't act like it's associated
    with a control, more like it's associated with the mouse.)"""
    def __init__(self, wxparent):
        self.wxparent = wxparent
        self.timer = None
    def set(self, msg, x, y):
        # Dispense with any old timers
        self.cancel()
        def on_timer_(wxparent, x, y):
            x, y = self.wxparent.ClientToScreenXY(x, y)
            tipwindow = wx.TipWindow(wxparent, msg)
            tipwindow.SetBoundingRect(wx.Rect(x, y, 10, 10))
        self.timer = wx.CallLater(500, on_timer_, self.wxparent, x, y)
    def cancel(self):
        if self.timer is not None:
            self.timer.Stop()
            self.timer = None
    def start(self): pass
    def stop(self):
        self.cancel()


class EventSource(events.EventSource):
    """Adapts WX events to low-level Rayon events."""
    # Note: handlers is a static variable defined below the handler
    # functions

    def __init__(self, chart, wx_panel):
        self.chart = chart
        self.event_source = wx_panel
        self.is_active = False
        self.subscriptions = {}
        # When the mouse moves, note its position so we know where it
        # was (more or less) when it's time to show a tooltip.
        self.last_mouse_position = None

        handlers = (
            (wx.EVT_LEFT_DOWN, self.on_mouse_left_down_),
            (wx.EVT_LEFT_UP, self.on_mouse_left_up_),
            (wx.EVT_LEFT_DCLICK, self.on_mouse_left_dclick_),
            (wx.EVT_RIGHT_DOWN, self.on_mouse_right_down_),
            (wx.EVT_RIGHT_UP, self.on_mouse_right_up_),
            (wx.EVT_RIGHT_DCLICK, self.on_mouse_right_dclick_),
            (wx.EVT_MOTION, self.on_mouse_motion_),
            (wx.EVT_IDLE, self.on_idle_))
        
        for wx_evt_type, handler in handlers:
            wx_panel.Bind(wx_evt_type, handler)
        # Tooltips
        self.popper = Popper(self.event_source)
        self.popper.start()
        self.tooltip_handled = False
    def activate(self):
        self.is_active = True
    def deactivate(self):
        self.is_active = False
    def subscribe(self, evt_name, handler):
        # subscribe() is called by EventListeners to register interest
        # in evt_name
        handlers = self.subscriptions.setdefault(evt_name, [])
        handlers.append(handler)
    def handle_mouse(self, evt_name, evt):
        try:
            if not self.is_active:
                return
            x, y = evt.GetPositionTuple()
            components = self.chart.xy_to_components(x, y)
            for c, c_x, c_y in components:
                handlers = self.subscriptions.get(evt_name, [])
                for h in handlers:
                    rc = h(c, WxRayonEvent(evt, x_offset=c_x, y_offset=c_y))
                    # Note: if, say, one component in a LayeredLayout
                    # returns STOP, it will prevent the execution of
                    # other events attached to that component , but
                    # not other events attached to other components in
                    # the layout.
                    #
                    # TODO: maybe introduce an
                    # events.STOP_NO_REALLY_I_MEAN_IT that stops all
                    # event processing.
                    if rc == events.STOP:
                        break
        finally:
            # TODO: When should I not let the rest of WX process this
            # event?
            evt.Skip()
    def handle_tooltip(self, evt):
        try:
            if (not self.is_active or
                self.last_mouse_position is None or 
                self.tooltip_handled):
                return
            msg, weight = None, 0
            # Find components interested in emitting a tooltip. Show
            # the "most important" one.
            x, y = self.last_mouse_position
            components = self.chart.xy_to_components(x, y)
            for c, c_x, c_y in components:
                handlers = self.subscriptions.get('tooltip', [])
                for h in handlers:
                    e = WxRayonEvent(evt, x=x, y=y, x_offset=c_x, y_offset=c_y)
                    rc = h(c, e)
                    curr_msg, curr_weight = e.get_tooltip()
                    if curr_weight is None:
                        continue
                    if curr_weight >= weight:
                        msg = curr_msg
                        weight = curr_weight
                    if rc == events.STOP:
                        break
            if msg is not None:
                self.popper.set(msg, x, y)
        finally:
            self.last_mouse_position = None
            self.tooltip_handled = True
            evt.Skip()
    # The WX event handlers
    def on_mouse_left_down_(self, evt):
        self.handle_mouse('mouse_left_down', evt)
    def on_mouse_left_up_(self, evt):
        self.handle_mouse('mouse_left_up', evt)
    def on_mouse_left_dclick_(self, evt):
        self.handle_mouse('mouse_left_dclick', evt)
    def on_mouse_right_down_(self, evt):
        self.handle_mouse('mouse_right_down', evt)
    def on_mouse_right_up_(self, evt):
        self.handle_mouse('mouse_right_up', evt)
    def on_mouse_right_dclick_(self, evt):
        self.handle_mouse('mouse_right_dclick', evt)
    def on_mouse_motion_(self, evt):
        if not self.is_active:
            evt.Skip()
            return
        self.last_mouse_position = evt.GetPositionTuple()
        self.popper.cancel()
        self.tooltip_handled = False
        self.handle_mouse('mouse_motion', evt)
    def on_idle_(self, evt):
        self.handle_tooltip(evt)



#
# Drawing context
#


# I'm making the effort here, but this really doesn't do anything for
# GTK, even though it's Cairo-based and this should work. I suspect it
# has something to do with a failed attempt to take care of this
# matter in wxwidgets. See http://trac.wxwidgets.org/ticket/4187.
#
# This does work if the target window is the only thing in the frame,
# so it's not a total loss. (At this writing, it works perfectly on
# Windows, BTW.)
if wx.Platform == '__WXGTK__' or wx.Platform == '__WXMSW__':
    def plat_round(x):
        # Round to nearest .5
        x = round(x * 2)
        if x % 2 == 0:
            x += 1
        return x / 2.0
else:
    plat_round = round

def fixup_line_for_raster(x1, y1, x2, y2):
    # Fix up both x and y, so independently drawn vertical and
    # horizontal lines will join properly at the ends
    if x1 == x2 or y1 == y2:
        return [plat_round(_) for _ in (x1, y1, x2, y2)]


class DebugGraphicsContext(backend.Context):
    # Proxy of GraphicsContext object that prints the method being called
    # out to the console.
    def __init__(self, dc):
        self.real = wx.GraphicsContext.Create(dc)
    def __getattr__(self, name):
        try:
            pprint
        except NameError:
            import pprint
        if hasattr(self.real, name):
            attr = getattr(self.real, name)
            def wrapper(*args, **kwargs):
                print "wx.GraphicsContext: Called %s" % name
                if len(args):
                    pprint.pprint(args)
                if len(kwargs):
                    pprint.pprint(kwargs)
                return attr(*args, **kwargs)
            return wrapper
        else:
            raise AttributeError(name)


class RealContext(backend.Context):
    """An abstraction over wx.DrawingContext/wx.GraphicsContext. """
    def __init__(self, dc):
        self.dc = dc
        self.gc = wx.GraphicsContext.Create(dc)
        #self.gc = DebugGraphicsContext(dc)
        self.pens = None
        self.brushes = None
        self.fonts = []
        self.interactive = interactive
        self.source_units = backend.Pixels
    def canvas_dimensions(self):
        return self.dc.GetSizeTuple()
    def normalize_dimensions(self, width, height):
        w, h = self.canvas_dimensions()
        if width is None:
            width = w
        if height is None:
            height = h
        return width, height
    # Transformation matrix stuff
    def translate(self, dx, dy, fixup=False):
        if fixup:
            dx, dy = tuple(plat_round(_) for _ in (dx, dy))
        return self.gc.Translate(dx, dy)
    def scale(self, dx, dy):
        return self.gc.Scale(dx, dy)
    # Resource stacks
    def push_state(self):
        return self.gc.PushState()
    def pop_state(self):
        return self.gc.PopState()
    def init_pen(self):
        if self.pens is None:
            self.pens = []
            self.pens.append(wx.Pen((0, 0, 0, 0)))
    def _push_pen(self, new_pen):
        self.init_pen()
        self.pens.append(new_pen)
        self.gc.SetPen(new_pen)
    def _pop_pen(self):
        self.init_pen()
        popped_pen = self.pens.pop(-1)
        curr_pen = self.pens[-1]
        self.gc.SetPen(curr_pen)
    def init_brush(self):
        if self.brushes is None:
            self.brushes = []
            self.brushes.append(wx.Brush((0, 0, 0, 0)))
    def _push_brush(self, new_brush):
        self.init_brush()
        self.brushes.append(new_brush)
        self.gc.SetBrush(self.gc.CreateBrush(new_brush))
    def _pop_brush(self):
        self.init_brush()
        popped_brush = self.brushes.pop(-1)
        curr_brush = self.brushes[-1]
        self.gc.SetBrush(self.gc.CreateBrush(curr_brush))
    def _push_pen_and_brush(self, new_pen, new_brush):
        self._push_pen(new_pen)
        try:
            self._push_brush(new_brush)
        except:
            try:
                self._pop_pen()
            except:
                pass
            raise
    def _pop_pen_and_brush(self):
        self._pop_pen()
        self._pop_brush()
    def _push_font(self, new_font):
        # Textbook case of why this class exists... :S
        self.fonts.append(self.dc.GetFont())
        self.gc.SetFont(new_font)
    def _pop_font(self):
        self.gc.SetFont(self.fonts.pop(-1))
    # Drawing proxies
    def draw_lines(self, segments, color, width, fixup=False, dashes=None,
                   dash_offset=0):
        if dashes is not None:
            new_pen = wx.Pen(color, width, wx.USER_DASH)
            # Rotate dash pattern to offset
            if dash_offset > 0:
                dash_offset = dash_offset % len(dashes)
                dashes = dashes[dash_offset:] + dashes[:dash_offset]
            dashes = [int(d) for d in dashes]
            new_pen.SetDashes(dashes)
        else:
            new_pen = wx.Pen(color, width, wx.SOLID)
        self._push_pen(new_pen)
        try:
            for x1, y1, x2, y2 in segments:
                if fixup:
                    x1, y1, x2, y2 = fixup_line_for_raster(x1, y1, x2, y2)
                self.gc.StrokeLine(x1, y1, x2, y2)
        finally:
            self._pop_pen()
    def draw_line(self, x1, y1, x2, y2, color, width, fixup=False,
                  dashes=None, dash_offset=0):
        return self.draw_lines([[x1, y1, x2, y2]], color, width, fixup,
                               dashes, dash_offset)
    def draw_ellipse(self, x, y, width, height, stroke_color, fill_color,
                     stroke_width=1):
        w = width
        h = height
        self._push_pen_and_brush(wx.Pen(stroke_color, 1),
                                 wx.Brush(fill_color))
        try:
            return self.gc.DrawEllipse(x, y, w, h)
        finally:
            self._pop_pen_and_brush()
    def draw_polygon(self, vertices, stroke_width, stroke_color, fill_color,
                     fixup=False):
        if stroke_color is None:
            raise RuntimeError("stroke_color is None! WTF were you thinking?!")
        if fill_color is None:
            fill_color = (0, 0, 0, 0)
        if fixup:
            vertices = [tuple(plat_round(_) for _ in v) for v in vertices]
        else:
            vertices = list(vertices)
        path = self.gc.CreatePath()
        first = vertices.pop(0)
        path.MoveToPoint(*first)
        for v in vertices:
            path.AddLineToPoint(*v)
        path.AddLineToPoint(*first)
        new_pen = wx.Pen(stroke_color, stroke_width)
        new_brush = wx.Brush(fill_color)
        self._push_pen_and_brush(new_pen, new_brush)
        try:
            self.gc.FillPath(path)
            self.gc.StrokePath(path)
        finally:
            self._pop_pen_and_brush()
        return vertices
    def draw_rectangle(self, x, y, width, height,
                       stroke_width, stroke_color, fill_color, fixup=False):
        w = width
        h = height
        
        # (x, y) == "lower left" (If h < y, (x,y) will be upper left)
        # Put borders within bounding box
        #w -= stroke_width / 2
        #h += stroke_width / 2
        vertices = ((x,y), (x,y-h), (x+w,y-h), (x+w,y))
        return self.draw_polygon(vertices,
                                 stroke_width, stroke_color, fill_color,
                                 fixup)
    def font_families(self):
        return font_families.keys()
    def font_styles(self):
        return font_styles.keys()
    def font_weights(self):
        return font_weights.keys()
    def _get_font(self, size, family, style, weight):
        return wx.Font(
            size, font_families[family],
            font_styles[style], font_weights[weight])
    def draw_text(self, text, bbox, size, family, style, weight,
                  color=(0, 0, 0, 0), angle=0):
        x, y = bbox[0]
        # Angle is measured in degrees. Somewhere between wxPython
        # 2.8.4.0 (which ships with Mac OS X) and 2.8.9.1, they
        # switched DrawRotatedText to use radians instead of
        # degrees. Therefore...
        if wx.VERSION > (2, 8, 4):
            angle = math.radians(angle)
        font = self._get_font(size, family, style, weight)
        self._push_font(font)
        self.push_state()
        self._push_brush(wx.Brush(color))
        try:
            self.gc.DrawRotatedText(text, x, y, angle)
        finally:
            self.pop_state()
            self._pop_font()
            self._pop_brush()
    def get_text_dimensions(self, text, size, family, style, weight):
        font = self._get_font(size, family, style, weight)
        self._push_font(font)
        try:
            return self.gc.GetFullTextExtent(text)[:2]
        finally:
            self._pop_font()


class DebugContext(object):
    # Proxy of RealContext object that prints the method being called
    # out to the console.
    def __init__(self, dc):
        self.real = RealContext(dc)
    def __getattr__(self, name):
        try:
            pprint
        except NameError:
            import pprint
        if hasattr(self.real, name):
            attr = getattr(self.real, name)
            def wrapper(*args, **kwargs):
                print "rayon.Context: Called %s" % name
                if len(args):
                    pprint.pprint(args)
                if len(kwargs):
                    pprint.pprint(kwargs)
                return attr(*args, **kwargs)
            return wrapper
        else:
            raise AttributeError(name)


Context = RealContext
#Context = DebugContext


class Page(backend.Page):
    @classmethod
    def from_buffer(cls, buff):
        return BufferedPage(buff)

class BufferedPage(backend.Page):
    """
    A :class:`~rayon.backend.Page` object that renders charts to a
    memory buffer.
    """
    def __init__(self, buff):
        self.buff = buff
    def write(self, chart):
        dc = wx.BufferedDC(None, self.buff)
        ctx = Context(dc)
        chart.layout(ctx)
            
    
