import socket
import struct

def inet_atoi(addr):
    """
    Returns an integer representation of the given dotted-quad string
    IPv4 address.
    """
    try:
        return int(struct.unpack('!L', socket.inet_aton(addr))[0])
    except socket.error:
        raise

def inet_itoa(num):
    """
    Returns a dotted-quad string representation of the given integer
    IPv4 address
    """
    return socket.inet_ntoa(struct.pack('!L', num))

class IPv4(object):
    """Datatype class for IPv4 addresses."""
    def __init__(self, ip_rep):
        if type(ip_rep) == str:
            self.ip = inet_atoi(ip_rep)
        else:
            self.ip = int(ip_rep)
    def __repr__(self): return "<IPv4: %s at 0x%x>" % (str(self), id(self))
    def __str__(self): return inet_itoa(self.ip)
    def __cmp__(self, other): return cmp(self.ip, other.ip)
    def __eq__(self, other): return self.ip == other.ip
    def __hash__(self): return hash(self.ip)

__all__ = ['IPv4']
