#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division

import pdb

try:
    import netsa.data.times
    import netsa.data.nice
except ImportError:
    raise RuntimeError("Couldn't locate netsa-python.")

import datetime

import data
import charts
import plots
import borders
import markers
import text
import ticks
import scales
import miscutils
import datautils
import backgrounds
import grids

import shortcuts


import time


def add_chart_subscriptions(chart, xborder, yborder, multiplot):
    def on_click(obj, evt):
        print "clicked: %s" % obj
    def on_click_pos(plot, evt):
            print "left down %s: %s,%s" % (plot, evt.GetX(), evt.GetY())
    def on_motion(obj, evt):
        if evt.LeftIsDown():
            print "left dragging in %s: %s,%s" % (
                obj, evt.GetX(), evt.GetY())
    def on_unclick(obj, evt):
        print "left up in %s: %s, %s" % (obj, evt.GetX(), evt.GetY())

    xborder.subscribe("mouse_left_down", on_click)
    yborder.subscribe("mouse_left_down", on_click)
    multiplot.subscribe("mouse_left_down", on_click_pos)
    multiplot.subscribe("mouse_left_up", on_unclick)
    multiplot.subscribe("mouse_motion", on_motion)
        



def dots_and_lines(backend, dots_fname, lines_fname):


    # The data object contains the input data, in a way that
    # makes it easy to dole out to various rendering
    # components.
    scatter_data = data.Dataset.from_file(dots_fname)


    # The first Plot we define is a simple scatterplot.
    scatter_plot = plots.ScatterPlot()
    
    # Each Plot has a set of axes on which points can be
    # plotted. A Scatterplot can plot points by X (position),
    # Y (position), marker color, marker shape and marker
    # size.
    #
    # Each axis is associated with a one-dimensional data
    # source (for input) and at least one Scale which
    # translates the input value into a value relevant to the
    # axis (i.e., a value in linear, color, size, etc.,
    # space -- whatever space the axis "expects").
    #
    # First, we do X and Y, which are spatial scales. The data
    # for X and Y will be mapped on scales that are somewhat
    # larger than the min and max of the data, so that the min
    # and max are aesthetically pleasing, "nice" numbers. (To
    # get them, we get statistics on the data from
    # scatter_data.)
    x_nice_tick_pos =  netsa.data.nice.nice_ticks(
       scatter_data.column(0).statistics().min(),
       scatter_data.column(0).statistics().max())
    x_lo, x_hi = x_nice_tick_pos[0], x_nice_tick_pos[-1]
    y_nice_tick_pos = netsa.data.nice.nice_ticks(
        scatter_data.column(1).statistics().min(),
        scatter_data.column(1).statistics().max())
    y_lo, y_hi = y_nice_tick_pos[0], y_nice_tick_pos[-1]

    # Scales themselves are callables that take a single value
    # and return a transformed value. LinearScales convert the
    # input data to a value between 0 and 1, representing the
    # proportion of the input value in the output range.
    #
    # (Given that we're overlaying another plot from another data
    # source later, it would be smart to make the scales large enough
    # to accomodate either, but that would complicate the example.)
    xscale = scales.LinearScale(x_lo, x_hi)
    yscale = scales.LinearScale(y_lo, y_hi)

    # Now, associate columns in the data and scales with axes.
    # The marker shape, size and color will always be the
    # same, regardless of input. Instead of creating Scale
    # objects, we can use lambdas
    scatter_plot.set_scales(x=xscale,
                            y=yscale,
                            marker_color=lambda _: (0, 0, 0),
                            marker_shape=lambda _: "dot",
                            marker_size=lambda  _: 1)
    # Marker color, shape and size don't have data columns
    # associated with them. The plot will pass None in as the
    # input argument (which the scale function will ignore).
    scatter_plot.set_data(x=scatter_data.column(0),
                          y=scatter_data.column(1))


    # Now, let's define a line plot.
    #line_data = datautils.ols_regression(plot_data.column(0),
    #                                     plot_data.column(1))
    line_data = data.Dataset.from_file(lines_fname)

    
    line_plot = plots.LinePlot()
    line_plot.set_scales(x=xscale, y=yscale,
                         line_color=lambda _: (255, 0, 0),
                         line_width=lambda _: 1,
                         line_style=lambda _: "solid")
    line_plot.set_data(x=line_data.column(0), y=line_data.column(1))
    

    # For a boxplot:
    #create_boxplot(plot_data, yscale)
    
    # The Chart object manages the overall layout of the
    # chart -- where the plot, borders, etc., go, where
    # padding goes, etc.
    chart = charts.SquareChart(backend)
    # Add the scatterplot and line plot to the chart, using a Multiplot
    multiplot = plots.Multiplot()
    multiplot.add(scatter_plot)
    multiplot.add(line_plot)
    chart.set_plot(multiplot)


    # The bottom border has a line style and contains one set
    # of tick marks
    xborder = borders.create('hline')
    chart.add_bottom_border(xborder, height=.1, tpad=.05)
    xtickmarker = markers.LabeledMarker(
        markers.create("vline", below=0, above=2),
        label=text.Labeler(font_size="normal",
                           halign="center", valign="bottom"),
        label_position="s")
    xtick_tuples = (
        (scatter_data.column(0).statistics().min(), None),
        (scatter_data.column(0).statistics().percentile(50), None),
        (scatter_data.column(0).statistics().max(), None),
    )
    xticks = ticks.Ticks(
        xtick_tuples, xscale, xtickmarker,
        formatter=ticks.create_formatter("string", "%s"))
    # The left border also has a line style and also contains
    # one set of tick marks
    yborder = borders.create('vline')
    chart.add_left_border(yborder, width=.1, rpad=.05)
    ytickmarker = markers.LabeledMarker(
        markers.create("hline", before=0, after=2),
        label=text.Labeler(font_size="normal",
                           halign="right", valign="center"),
        label_position="e")
    ytick_tuples = (
        (scatter_data.column(1).statistics().min(), None),
        (scatter_data.column(1).statistics().percentile(50), None),
        (scatter_data.column(1).statistics().max(), None),
    )
    yticks = ticks.Ticks(
        ytick_tuples, yscale, ytickmarker,
        formatter=ticks.create_formatter("string", "%s"))


    # The x border will use the x ticks to identify places to
    # put tick marks, and will position them using the x
    # scale. Ditto for the y border, ticks and scale.
    xborder.add_tickset(xticks)
    yborder.add_tickset(yticks)

    # Add null borders on all sides to provide padding
    chart.add_left_border(borders.create('none'), width=.05)
    chart.add_right_border(borders.create('none'), width=.05)
    chart.add_top_border(borders.create('none'), height=.05)
    chart.add_bottom_border(borders.create('none'), height=.05)


    # Add events to chart
    add_chart_subscriptions(chart, xborder, yborder, multiplot)
    return chart



def timeseries(backend, fname, outlier_thresh=95):
    # For more exhaustive documentation, look at
    # dots_and_lines. I'll only be documenting the new bits
    # here.

    # Data
    print "Getting dataset from file...",
    _start = time.time()
    ts_data = data.Dataset.from_file(
        fname,
        typemap=[netsa.data.times.make_datetime,float])
    _end = time.time()
    print str(_end - _start)


    print "Filtering outliers...",
    _start = time.time()
    thresh = ts_data.column(1).statistics().percentile(outlier_thresh)
    outlier_data, ts_data = ts_data.filter_both(lambda r: r[1] >= thresh)
    _end = time.time()
    print str(_end - _start)

    print "Smoothing data...",
    _start = time.time()
    smoothed_data = datautils.moving_average(ts_data, window=600)
    _end = time.time()
    print str(_end - _start)

    # Information for tick marks
    x_nice_tick_pos = netsa.data.nice.nice_time_ticks(
        ts_data.column(0).statistics().min(),
        ts_data.column(0).statistics().max())
    x_lo, x_hi = x_nice_tick_pos[0], x_nice_tick_pos[1]
    # TODO: nice_ticks and nice_time_ticks should return the same
    # thing. Right now, nice_ticks returns a sequence, and
    # get_time_ticks returns a (start, stop, iterator) tuple.
    y_nice_tick_pos = miscutils.nice_ticks(
        ts_data.column(1).statistics().min(),
        ts_data.column(1).statistics().max())
    y_lo, y_hi = y_nice_tick_pos[0], y_nice_tick_pos[-1]

    
    
    # Scales
    xscale = scales.TimeScale(x_lo, x_hi)
    yscale = scales.LinearScale(y_lo, y_hi)


    # The dots
    dot_plot = plots.ScatterPlot()
    dot_plot.set_scales(x=xscale,
                        y=yscale,
                        marker_color=lambda _: (64, 64, 64),
                        marker_shape=lambda _: "dot",
                        marker_size=lambda  _: 1)

    dot_plot.set_data(x=ts_data.column(0),
                      y=ts_data.column(1))

    # The "bings"
    bing_plot = plots.ScatterPlot()
    bing_plot.set_scales(x=xscale,
                         y=lambda _: 1.0,
                         marker_color=lambda _: (192,0,0),
                         marker_shape=lambda _: "vline",
                         marker_size=lambda _: .5)

    bing_plot.set_data(x=outlier_data.column(0),
                       y=outlier_data.column(1))

    # The lines
    line_plot = plots.LinePlot()
    line_plot.set_scales(x=xscale, y=yscale,
                         line_color=lambda _: (0, 0, 192),
                         line_width=lambda _: 2,
                         line_style=lambda _: "solid")
    
    line_plot.set_data(x=smoothed_data.column(0),
                       y=smoothed_data.column(1))



    # Borders and tick marks
    xtickmarker = markers.LabeledMarker(
        markers.create("vline", below=0, above=2),
        label=text.Labeler(font_size="normal",
                           halign="center", valign="bottom"),
        label_position="s")
    xticks = ticks.Ticks(
        [(x, None) for x in x_nice_tick_pos[2]],
        xscale, xtickmarker,
        formatter=ticks.create_formatter("datetime", "%H:%M"))
    xborder = borders.create('hline')
    xborder.add_tickset(xticks)

    ytickmarker = markers.LabeledMarker(
        markers.create("hline", before=0, after=2),
        label=text.Labeler(font_size="normal",
                           halign="right", valign="center"),
        label_position="e")
    yticks = ticks.Ticks(
        [(y, None) for y in y_nice_tick_pos],
        yscale, ytickmarker,
        formatter=ticks.create_formatter("string", "%.2f"))
    yborder = borders.create('vline')
    yborder.add_tickset(yticks)
        
    # Add plots to the chart
    chart = charts.SquareChart(backend)
    multiplot = plots.Multiplot()
    multiplot.add(dot_plot)
    multiplot.add(line_plot)
    multiplot.add(bing_plot)
    chart.set_plot(multiplot)
    
    chart.add_bottom_border(xborder, height=.1, tpad=.05)
    chart.add_left_border(yborder, width=.1, rpad=.05)
    # Add null borders on all sides to provide padding
    chart.add_left_border(borders.create('none'), width=.05)
    chart.add_right_border(borders.create('none'), width=.05)
    chart.add_top_border(borders.create('none'), height=.05)
    chart.add_bottom_border(borders.create('none'), height=.05)


    # Add events to chart
    add_chart_subscriptions(chart, xborder, yborder, multiplot)
    return chart

def existence(backend, fname, lo, med, hi):
    # columns are: date, port, volume
    ts_data = data.Dataset.from_file(
        fname,
        typemap=[netsa.data.times.make_datetime,int,float])

    # Ignore stuff below our lowest threshold
    ts_data = ts_data.filter_pass(lambda r: r[2] >= lo)

    # Tick positions
    x_nice_tick_pos = netsa.data.nice.get_time_ticks(
        ts_data.column(0).statistics().min(),
        ts_data.column(0).statistics().max())
    x_lo, x_hi = x_nice_tick_pos[0], x_nice_tick_pos[1]
    y_lo, y_hi = 0, 65535

    # Scales
    xscale = scales.TimeScale(x_lo, x_hi)
    yscale = scales.LogScale(y_lo, y_hi)

    # Dots
    def marker_color_scale(x):
        if x > hi:    return (255, 0, 0, 255)
        elif x > med: return (255, 255, 0, 128)
        elif x >= lo: return (0, 255, 0, 64)

    existence_plot = plots.ScatterPlot()
    existence_plot.set_scales(x=xscale,
                        y=yscale,
                        marker_color=marker_color_scale,
                        marker_shape=lambda _: "dot",
                        marker_size=lambda  _: 1)

    existence_plot.set_data(x=ts_data.column(0),
                      y=ts_data.column(1),
                      marker_color=ts_data.column(2))


    # Borders and tick marks
    xtickmarker = markers.LabeledMarker(
        markers.create("vline", below=0, above=2),
        label=text.Labeler(font_size="normal",
                           halign="center", valign="bottom"),
        label_position="s")
    xticks = ticks.Ticks(
        [(x, None) for x in x_nice_tick_pos[2]],
        xscale, xtickmarker,
        formatter=ticks.create_formatter("datetime", "%H:%M"))
    xborder = borders.create('hline')
    xborder.add_tickset(xticks)

    ytickmarker = markers.LabeledMarker(
        markers.create("hline", before=0, after=2),
        label=text.Labeler(font_size="normal",
                           halign="right", valign="center"),
        label_position="e")
    yticks = ticks.Ticks(
        [(y, None) for y in
         (0, 21, 25, 53, 80, 443, 1024,
          5000, 10000, 20000, 30000, 40000, 50000, 65535)],
        yscale, ytickmarker,
        formatter=ticks.create_formatter("string", "%d"))
    yborder = borders.create('vline')
    yborder.add_tickset(yticks)


    chart = charts.SquareChart(backend)
    multiplot = plots.Multiplot()
    multiplot.add(existence_plot)
    chart.set_plot(multiplot)

    chart.add_bottom_border(xborder, height=.1, tpad=.05)
    chart.add_left_border(yborder, width=.1, rpad=.05)
    # Add null borders on all sides to provide padding
    chart.add_left_border(borders.create('none'), width=.05)
    chart.add_right_border(borders.create('none'), width=.05)
    chart.add_top_border(borders.create('none'), height=.05)
    chart.add_bottom_border(borders.create('none'), height=.05)


    # Add events to chart
    add_chart_subscriptions(chart, xborder, yborder, multiplot)
    return chart


def cloud_chamber(backend, data_fname):
    print "eating file"
    f = open(data_fname, 'r')
    max_y = 0
    try:
        series = [[int(c) for c in line.split(" ")]
                  for line in f.readlines()]
        max_y = max(max(s) for s in series)
    finally:
        f.close()
    print "building scales"
    x_data = [x for x in xrange(48)]
    xscale = scales.CategoricalScale(x_data)
    y_nice_tick_pos =  miscutils.nice_ticks(0, max_y)
    #yscale = scales.LinearScale(y_nice_tick_pos[0], y_nice_tick_pos[-1])
    yscale = scales.LogScale(y_nice_tick_pos[0], y_nice_tick_pos[-1])
    print "adding borders"
    ytickmarker = markers.LabeledMarker(
        markers.create("hline", before=0, after=2),
        label=text.Labeler(font_size="small",
                           halign="right", valign="center"),
        label_position="e")
    yticks = ticks.Ticks(
        [(y, None) for y in y_nice_tick_pos],
        yscale, ytickmarker,
        formatter=ticks.create_formatter("string", format_str="%d"))
    yborder = borders.create("v")
    yborder.add_tickset(yticks)
    print "building plots"
    mplot = plots.Multiplot()
    x_data = [x for x in xrange(48)]
    for row in series:
        d = data.Dataset(zip(x_data, row),
                         ['bin', 'hcount'],
                         typemap=[int, int])
        plot = plots.LinePlot()
        plot.set_data(x=d.column('bin'), y=d.column('hcount'))
        plot.set_scales(x=xscale, y=yscale,
                        line_color=(0, 0, 0, 8),
                        line_width=1, line_style='solid')
        mplot.add(plot)
    print "adding to chart"
    chart = charts.SquareChart(backend)
    chart.set_plot(mplot)
    chart.add_left_border(yborder, width=.05)
    # Add null borders on all sides to provide padding
    chart.add_left_border(borders.create('none'), width=.05)
    chart.add_right_border(borders.create('none'), width=.05)
    chart.add_top_border(borders.create('none'), height=.05)
    chart.add_bottom_border(borders.create('none'), height=.05)
    print "all done"
    return chart

def barchart(backend, bars_fname):
    bar_data = data.Dataset.from_file(bars_fname, typemap=[str, float])
    bar_plot = plots.BarPlot()
    # Tick positions and scale limits
    x_tick_pos = bar_data.column(0)
    ystats = bar_data.column(1).statistics()
    if ystats.min() < 0:
        y_lo = ystats.min()
    else:
        y_lo = 0
        
    if ystats.max() < 0:
        y_hi = 0
    else:
        y_hi = ystats.max()

    y_tick_pos = miscutils.nice_ticks(y_lo, y_hi)
    # Re-evaluate y_[lo|hi] in light of the nice tick values
    if y_hi < y_tick_pos[-1]:
        y_hi = y_tick_pos[-1]
    if y_lo > y_tick_pos[0]:
        y_lo = y_tick_pos[0]
    
    # Scales
    xscale = scales.CategoricalScale(x_tick_pos, lo=.05, hi=.95)
    yscale = scales.LinearScale(y_lo, y_hi)
    #yscale = scales.LogScale(y_lo, y_hi)
    # Set scales
    bar_plot.set_scales(x=xscale, y=yscale,
                        bar_color=(0, 0, 255),
                        bar_width=xscale.interval() * .85,
                        border_color = (0, 0, 0),
                        border_width=1,
                        bar_align=-.5)
    # Set data
    bar_plot.set_data(x=bar_data.column(0), y=bar_data.column(1))

    # The chart
    chart = charts.SquareChart(backend)
    multiplot = plots.Multiplot()
    multiplot.add(bar_plot)
    chart.set_plot(multiplot)

    # Ticks
    xtickmarker = markers.LabeledMarker(
        markers.create("vline", below=0, above=2),
        label=text.Labeler(font_size="normal",
                           halign="center", valign="bottom"),
        label_position="s")
    xticks = ticks.Ticks(
        tuple(e for e in enumerate(x_tick_pos)), xscale, xtickmarker,
        formatter=ticks.create_formatter("string", "%s"))
    ytickmarker = markers.LabeledMarker(
        markers.create("hline", before=0, after=2),
        label=text.Labeler(font_size="normal",
                           halign="right", valign="center"),
        label_position="e")
    yticks = ticks.Ticks(
        tuple((y, None) for y in y_tick_pos),
        yscale, ytickmarker,
        formatter=ticks.create_formatter("string", "%.2f"))
    # Borders
    xborder = borders.create('hline')
    xborder.add_tickset(xticks)
    chart.add_bottom_border(xborder, height=.1, tpad=.05)
    yborder = borders.create('vline')
    yborder.add_tickset(yticks)
    chart.add_left_border(yborder, width=.1, rpad=.05)

    # Padding
    chart.add_left_border(borders.create('none'), width=.05)
    chart.add_right_border(borders.create('none'), width=.05)
    chart.add_top_border(borders.create('none'), height=.05)
    chart.add_bottom_border(borders.create('none'), height=.05)

    # Events
    def on_motion_(plot, evt):
        bars = plot.bars_at(evt.x(), evt.y())
        if len(bars) == 0:
            chart.tooltip(None)
        else:
            chart.tooltip("flurble! %s" % str(bars[0]))

    if backend.interactive:
        # todo: add events to chart
        bar_plot.subscribe("mouse_motion", on_motion_)

    # Finally...
    return chart


def horizontal_barchart(backend, bars_fname):
    # This example uses the shortcuts api to show how you can reduce
    # boilerplate.
    bar_data = data.Dataset.from_file(bars_fname, typemap=[str, float, int])
    bar_plot = plots.HorizontalBarPlot()
    # c == "categorical axis" == the axis on which categories are
    # plotted (Y, in this example)
    #
    # v == "value axis" == the axis on which values are plotted
    # (X, in this example)
    
    # Tick positions and scale limits
    c_tick_pos = bar_data.column(0)
    v_stats = bar_data.column(1).statistics()
    v_lo = min(v_stats.min(), 0)
    v_hi = max(v_stats.max(), 0)
    v_tick_pos = miscutils.nice_ticks(v_lo, v_hi)
    # Our scale has changed, so adjust v_[lo|hi]
    v_hi = max(v_hi, v_tick_pos[1])
    v_lo = min(v_lo, v_tick_pos[0])
    # Scales
    c_scale = scales.CategoricalScale(c_tick_pos, lo=.10, hi=.95)
    #v_scale = scales.LogScale(v_lo, v_hi)
    v_scale = scales.LinearScale(v_lo, v_hi)
    bar_plot.set_scales(x=v_scale, y=c_scale,
                        bar_color=(0, 0, 255, 128),
                        bar_width=c_scale.interval(),
                        border_color = (0, 0, 0),
                        border_width=1,
                        bar_align=-.5)
    # Data
    bar_plot.set_data(x=bar_data.column(1), y=bar_data.column(0))
    # Meta-data
    if bar_data.numcols() > 2:
        meta_col = bar_data.column(2)
        bar_plot.set_metadata(**{meta_col.name: meta_col})
    # Chart
    chart = charts.SquareChart(backend)
    multiplot = plots.Multiplot()
    multiplot.add(bar_plot)
    chart.set_plot(multiplot)
    # Ticks
    c_tickseq = tuple(e for e in enumerate(c_tick_pos))
    c_ticks = shortcuts.string_ticks(c_tickseq, c_scale,
                                     shortcuts.left_tickmarker(), "%s")
    v_tickseq = tuple((x, None) for x in v_tick_pos[2])
    v_tickmarker = shortcuts.bottom_tickmarker(valign="center",
                                               halign="right", angle=90)
    v_ticks = shortcuts.string_ticks(v_tickseq, v_scale, v_tickmarker, "%.2f")
    # Borders
    xborder = borders.create("hline")
    xborder.add_tickset(v_ticks)
    chart.add_bottom_border(xborder, height=.1, tpad=.05)
    yborder = borders.create("vline")
    yborder.add_tickset(c_ticks)
    chart.add_left_border(yborder, width=.1, rpad=.05)
    # Titling
    title_lbl = text.StaticText("Sample title",
                            font_size="x-large", halign="center",
                            )#border_bottom="1 solid 000000")
    # The halign in the StaticText refers to how the text will be
    # aligned relative to the x,y coordinate where it is drawn. This
    # halign refers to the whole box's position relative to the
    # border.
    title = borders.create("labeled")
    title.set_text(title_lbl, halign="center", valign="center")
    chart.add_top_border(title, height=.02, tpad=.01, bpad=.03)
    # Padding
    chart.add_padding("all", size=.05)
    # Give the plot a gray background with a black outline
    chart.set_plot_background(backgrounds.ColorBackground(
        fill_color=(204,204,204), stroke_color=(0,0,0)))
    # Add gridlines for each tickmark on y. GridLines are just like
    # Plots -- in fact, they're a special kind of plot. 
    hlines = grids.HorizontalGridLines()
    # Synthesize a dataset for the grid markers, which should be in
    # the middle of every bar. (This works as long as the bar is
    # centered between the positions on the categorical scale. We'd
    # need to do more work if there is some asymmetric padding
    # somewhere.)
    #hline_c_scale = scales.CategoricalScale(c_tick_pos, lo=.10, hi=.90)
    hline_c_scale = c_scale
    hlines.set_data(pos=c_tick_pos)
    hlines.set_scales(pos=hline_c_scale, line_style='solid',
                      line_color=(0, 0, 0, 100), line_width=1)
    chart.add_rear_gridlines(hlines)

    
    
    # Events
    def on_motion_(plot, evt):
        bars = plot.bars_at(evt.x(), evt.y())
        if len(bars) == 0:
            chart.tooltip(None)
        else:
            chart.tooltip("flurble! %s" % str(bars[0]))

    if backend.interactive:
        # todo: add events to chart
        bar_plot.subscribe("mouse_motion", on_motion_)

    # Finally...
    return chart


    
    
    

    
