#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
import markers
import text
import ticks

"""Convenience functions for reducing boilerplate in chart
creation. These functions also serve as documentation for the
lower-level bits of the API."""

def top_tickmarker(marker_style="vline", font_size="normal", angle=0):
    """Create a basic tickmarker for the top border of a chart.

    @type marker_style: string
    @param marker_style: style of marker to use to mark ticks

    @type font_size: string
    @param font_size: Size of font to use for tick labels
    """
    return markers.LabeledMarker(
        markers.create(marker_style, below=0, above=2),
        label=text.Labeler(font_size=font_size,
                           halign="center", valign="top", angle=angle),
        label_position="n")


def bottom_tickmarker(marker_style="vline", font_size="normal",
                      halign="center", valign="bottom", angle=0):
    """Create a basic tickmarker for the bottom border of a chart.

    @type marker_style: string
    @param marker_style: style of marker to use to mark ticks

    @type font_size: string
    @param font_size: Size of font to use for tick labels
    """
    return markers.LabeledMarker(
        markers.create(marker_style, below=0, above=2),
        label=text.Labeler(font_size=font_size,
                           halign=halign, valign=valign, angle=angle),
        label_position="s")


def left_tickmarker(marker_style="hline", font_size="normal", angle=0):
    """Create a basic tickmarker for the left border of a chart.

    @type marker_style: string
    @param marker_style: style of marker to use to mark ticks

    @type font_size: string
    @param font_size: Size of font to use for tick labels
    """
    return markers.LabeledMarker(
        markers.create(marker_style, before=0, after=2),
        label=text.Labeler(font_size=font_size,
                           halign="right", valign="center", angle=angle),
        label_position="e")


def right_tickmarker(marker_style="hline", font_size="normal", angle=0):
    """Create a basic tickmarker for the right border of a chart.

    @type marker_style: string
    @param marker_style: style of marker to use to mark ticks

    @type font_size: string
    @param font_size: Size of font to use for tick labels

    @type angle: number
    @param angle: degrees of rotation from baseline (which is horizontal) """
    return markers.LabeledMarker(
        markers.create(marker_style, before=0, after=2),
        label=text.Labeler(font_size=font_size,
                           halign="left", valign="center", angle=angle),
        label_position="w")


def string_ticks(seq, scale, tickmarker, format):
    """Create a series of ticks on the supplied scale using the
    supplied sequence, marked using the supplied marker and labeled in
    the supplied format.

    @type seq: sequence
    @param seq: sequence of tuples suitable for creating a Ticks object.

    @type scale: scales.Scale
    @param scale: scale on which to map ticks

    @type tickmarker: markers.LabeledMarker
    @param tickmarker: marker with which to mark the scale

    @type formatter: ticks.TickFormatter
    @param formatter: Formatter with which to format the labels."""

    return ticks.Ticks(seq, scale, tickmarker,
                       formatter=ticks.create_formatter("string", format))


