#  Copyright (C) 2011 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Network Situational Awareness Portal and related source
#  code is subject to the terms of the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division
import sys
import pdb
import wx
import backend_wx

import samplecharts

import time


## Other Stuff ##########################################################


class MainFrame(wx.Frame):
    def __init__(self, title, size):
        wx.Frame.__init__(self, None, -1, title, size=size,
                          style=wx.DEFAULT_FRAME_STYLE)
        self.sz = wx.BoxSizer(wx.HORIZONTAL)
    def add_children(self, *children):
        for child in children:
            self.sz.Add(child, flag=wx.ALL | wx.GROW | wx.EXPAND)
        self.sz.Fit(self)
        


class PreviewPanel(wx.Panel):
    def __init__(self, parent, chart, size):
        wx.Panel.__init__(self, parent, size=size)
        self.chart = chart
        chart.set_parent(self)
        chart.register_subscriptions(self, backend_wx)
        self.Bind(wx.EVT_PAINT, self._evt_paint)


    def _evt_paint(self, evt):
        pdc = wx.PaintDC(self)
        ctx = backend_wx.Context(pdc)
        start = time.time()
        self.chart(ctx)
        end = time.time()
        #print "[%f]" % (end - start)

    def print_chart(self, fname):
        psdc = wx.PostScriptDC(wx.PrintData())
        self.chart(psdc)



class MatrixPanel(wx.Panel):
    "Holds other panels in a grid."
    def __init__(self, parent, cells):
        wx.Panel.__init__(self, parent)
        self.sz = wx.GridSizer(rows=len(cells[0]), cols=len(cells))
        for row in cells:
            for cell in row:
                cell.Reparent(self)
                self.sz.Add(cell)
        self.SetSizerAndFit(self.sz)

        

def read_input(fname):
    f = open(fname, 'r')
    data = []
    try:
        for line in f:
            data.append([float(i) for i in line.strip().split(",")])
    finally:
        f.close()
    return data



class ScatterApp(wx.App):
    def _bind_events(self):
        self.Bind(wx.EVT_CLOSE, self._evt_frame_close)

    
    def __init__(self):
        start = time.time()
        wx.App.__init__(self, redirect=False,
                        filename=None, useBestVisual=True)
        self.drawframe = MainFrame("FARK", (450, 340))

        # Note: The parent is irrelevant to the cells, they'll get
        # reparented when they're inserted into the MatrixPanel
        # The new way
        chart_functions = {
            "dots" : lambda : samplecharts.dots_and_lines(backend_wx,
                                                          sys.argv[2],
                                                          sys.argv[3]),
            "ts"   : lambda : samplecharts.timeseries(backend_wx,
                                                      sys.argv[2]),
            "exist": lambda : samplecharts.existence(backend_wx,
                                                     sys.argv[2],
                                                     lo=100,
                                                     med=1000,
                                                     hi=10000),
            "bars" : lambda : samplecharts.barchart(backend_wx,
                                                    sys.argv[2]),
            "hbars" : lambda : samplecharts.horizontal_barchart(backend_wx,
                                                                sys.argv[2])
        }
        #cells = [ [PreviewPanel(self.drawframe,
        #                        chart_functions[sys.argv[1]](),
        #                        (800, 800))
        #           for x in xrange(1)]
        #          for y in xrange(1) ]
        #cells = [[PreviewPanel(self.drawframe,
        #                       chart_functions[sys.argv[1]](), (800, 800))]]


        self.drawframe.add_children(PreviewPanel(
            self.drawframe, chart_functions[sys.argv[1]](), (400, 400)))

        self._bind_events()



        
        self.drawframe.Show()
        self.SetTopWindow(self.drawframe)
        end = time.time()

        print end - start


    def quit(self):
        self.drawframe.Close()
        self.Exit()


    def _evt_frame_close(self, evt):
        self.quit()



def main():
    app = ScatterApp()
    app.MainLoop()

if __name__ == '__main__':
    main()
